﻿using System;
using System.Collections.Generic;

/// @author  Vesa Lappalainen
/// @version 27.10.2012   
/// <summary>
/// Esimerkkejä olioista
/// </summary>
public class ElaimetPerimalla
{
    /// <summary>
    /// Tutkitaan olioiden käyttäytymistä
    /// </summary>
    public static void Main()
    {
        Kissa miuku = new Kissa("Miuku", 900);
        Kissa mirri = new Kissa("Mirri", 19000);
        Koira musti = new Koira("Musti", 2700);
        Koira pluto = new Koira("Pluto", 500);

        miuku.Aantele();
        mirri.Aantele();
        musti.Aantele();
        Console.WriteLine(miuku); // kutsuu oikeasti miuku.ToString();
        Console.WriteLine(musti);

        List<Elain> elaimet = new List<Elain>();
        Elain[] elukatTaulukossa = { miuku, mirri, musti, pluto };
        elaimet.Add(elukatTaulukossa[0]);
        elaimet.Add(mirri);
        elaimet.Add(musti);
        elaimet.Add(pluto);

        foreach (Elain elain in elaimet)
            elain.Aantele();

        pluto.Juoksuvauhti = 50; // Javassa pluto.setJuoksuvauhti(59);
        musti.Kuolaa = true;

        foreach (Elain elain in elaimet)
            Console.WriteLine(elain);
    }
}


/// <summary>
/// Elain-luokka, joka osaa äännellä
/// </summary>
public class Elain
{
    private readonly string nimi;   // attribuutti
    private readonly double paino;

    /// <summary>
    /// Alustetaan eläimen nimi ja paino
    /// </summary>
    /// <param name="nimi">eläimelle annettava nimi</param>
    /// <param name="paino">eläimen alkuperäinen paino</param>
    protected Elain(string nimi, double paino) // muodostaja
    {
        this.nimi = nimi;
        this.paino = paino;
    }


    /// <summary>
    /// Yleinen eläin ei osaa äännellä
    /// </summary>
    public virtual void Aantele() // metodi
    {
        Console.WriteLine("???");
    }


    /// <summary>
    /// Eläimen tiedot merkkijonona
    /// </summary>
    /// <returns>Eläimen tiedot merkkijonona</returns>
    public override string ToString()
    {
        return nimi + " " + paino;
    }
}


/// <summary>
/// Kissa-luokka, joka osaa äännellä
/// </summary>
public class Kissa : Elain
{
    private readonly int hannanpituus;

    /// <summary>
    /// Alustetaan kissan nimi ja paino
    /// </summary>
    /// <param name="nimi">kissalle annettava nimi</param>
    /// <param name="paino">kissan alkuperäinen paino</param>
    public Kissa(string nimi, double paino) : base(nimi, paino)
    {
        hannanpituus = 20;
    }


    /// <summary>
    /// Ääntelee kuten kissa ääntelee
    /// </summary>
    public override void Aantele() // metodi
    {
        Console.WriteLine("Miuaa");
    }


    /// <summary>
    /// Kissan tiedot merkkijonona
    /// </summary>
    /// <returns>Kissan tiedot merkkijonona</returns>
    public override string ToString()
    {
        return base.ToString() + " " + hannanpituus;
    }

}

/// <summary>
/// Koira-luokka, joka osaa äännellä
/// </summary>
public class Koira : Elain
{
    private bool kuolaa;
    public bool Kuolaa
    {
        set { kuolaa = value; }   // pluto.Kuolaa = false;
        get { return kuolaa; }    // if ( pluto.Kuolaa ) ...
    }

    // Java vastine:
    //  pluto.setKuolaa(false);
    //  if ( pluto.getKuolaa() ) .

    public double Juoksuvauhti { get; set; } // luo automaattisesti vastaavan attribuutin

    /// <summary>
    /// Alustetaan koiran nimi ja paino
    /// </summary>
    /// <param name="nimi">koiralle annettava nimi</param>
    /// <param name="paino">koiran alkupeärinen paino</param>
    /// <param name="kuolaa">kuolaako koira vai ei</param>
    public Koira(string nimi, double paino, bool kuolaa = false) : base(nimi, paino)
    {
        this.Kuolaa = kuolaa;
        Juoksuvauhti = 40;
    }


    /// <summary>
    /// Ääntelee kuten koira ääntelee
    /// </summary>
    public override void Aantele() // metodi
    {
        Console.WriteLine("Hau hau");
    }


    /// <summary>
    /// Koiran tiedot merkkijonona
    /// </summary>
    /// <returns>Koiran tiedot merkkijonona</returns>
    public override string ToString()
    {
        return base.ToString() + " " + Kuolaa + " " + Juoksuvauhti;
    }
}


