﻿using System;
using Jypeli;
using Jypeli.Effects;
// ReSharper disable MemberCanBeProtected.Global

namespace HangryDog;

public class Efektit
{
    public static void Savuta(PhysicsGame peli, IPhysicsObject olio, bool useShock = false, double kerroin = 10)
    {
        Smoke savu = new Smoke();
        savu.MaximumLifetime = TimeSpan.FromSeconds(3);
        savu.Position = olio.Position;
        peli.Add(savu); olio.Destroy();
        // savu.FadeOut(5); // ei tuhoa oliota efektin jälkeen
        Timer timer = new Timer();
        timer.Interval = 3;
        timer.Start();
        timer.Timeout += delegate
        {
            savu.Destroy();
            timer.Stop();
        };
        //        savunAani.Play();
    }
}


public class Syotava : PhysicsObject
{
    public Syotava(PhysicsGame peli, double w, Vector p, string tunniste, int elinaika)
        : base(w, w, p.X, p.Y)
    {
        this.Image = Game.LoadImage(tunniste);
        this.Height = this.Width * this.Image.Height / this.Image.Width;
        peli.Add(this);
        this.Tag = tunniste;
        this.LifetimeLeft = new TimeSpan(10000 * elinaika);
    }
}


public class Luu : Syotava
{
    public Luu(PhysicsGame peli, double w, Vector p)
        : base(peli, w, p, "luu", 2000)
    {
        this.Destroyed += delegate { Efektit.Savuta(peli, this); };
    }
}


public class Juusto : Syotava
{
    public Juusto(PhysicsGame peli, double w, Vector p)
        : base(peli, w, p, "juusto", 50000)
    {
        this.Destroyed += delegate { Efektit.Savuta(peli, this); };
    }
}


public class Elain : PhysicsObject // PlatformCharacter
{
    private readonly string nimi;
    public string Nimi { get { return nimi;  } }

    private bool vasen ;

    public Elain(PhysicsGame peli, double w, Vector p, string nimi, string tunniste)
        : base(w, w, p.X, p.Y)
    {
        this.Image = Game.LoadImage(tunniste);
        this.Tag = tunniste;
        this.Height = this.Width * this.Image.Height / this.Image.Width;
        this.nimi = nimi;
        Label label = new Label(nimi);
        this.Add(label);
        label.RelativePosition = Vector.Zero;
        peli.Add(this);
        CanRotate = false;
    }

    public virtual void Liiku(Vector suunta)
    {
        //this.Hit(new Vector(0,suunta.Y));
        //Walk(suunta.X);
        this.Hit(suunta);
        bool nytVasen = suunta.X < 0;
        if (nytVasen ^ vasen) this.MirrorImage();
        vasen = nytVasen;
    }


    public virtual void Aantele()
    {

    }


    public virtual void Syo(PhysicsObject kuka, PhysicsObject kenet)
    {
        kenet.Destroy();
        this.Size += new Vector(10, 10);
        this.Aantele();
    }
}


public class Koira : Elain
{
    public Koira(PhysicsGame peli, double w, Vector p, string nimi = "")
        : base(peli, w, p, nimi, "koira")
    {
        peli.AddCollisionHandler(this, "luu", Syo);
       // peli.AddCollisionHandler(this, "hiiri", Syo);
    }


    public override void Aantele()
    {
        Game.PlaySound("KoiranAani1");
    }
}


public class Kissa : Elain
{
    public Kissa(PhysicsGame peli, double w, Vector p, string nimi = "")
        : base(peli, w, p, nimi, "kissa")
    {
        peli.AddCollisionHandler(this, "hiiri", Syo);
    }

    public override void Aantele()
    {
        Game.PlaySound("KissanAani01");
    }

}


public class Hiiri : Elain
{
    public Hiiri(PhysicsGame peli, double w, Vector p, Juusto juusto)
        : base(peli, w, p, "", "hiiri")
    {
        this.LifetimeLeft = new TimeSpan(0, 0, 0, 10);
        peli.AddCollisionHandler(this, "juusto", Syo);
        this.Destroyed += delegate { var _ =new Luu(peli, Width * 0.8, Position); };
        FollowerBrain brain = new FollowerBrain(juusto);
        this.Brain = brain;
    }

    public override void Syo(PhysicsObject kuka, PhysicsObject kenet)
    {
        base.Syo(kuka, kenet);
        this.LifetimeLeft = new TimeSpan(0, 0, 0, 15);
    }

    public override void Aantele()
    {
        Game.PlaySound("Naps");
    }
}
